//+------------------------------------------------------------------
//|
//| Derivative Oscillator as published by Constance Brown
//| "Technical Analysis for the trading professional"
//| Low lag oscillator which is useful at divergence analysis.
//+------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 6
#property indicator_color1  DeepSkyBlue
#property indicator_color2  DeepSkyBlue
#property indicator_color3  PaleVioletRed
#property indicator_color4  PaleVioletRed
#property indicator_color5  DimGray
#property indicator_color6  Red
#property indicator_width1  2
#property indicator_width4  2
#property indicator_width5  2
#property strict

//
//
//
//
//

extern ENUM_TIMEFRAMES    TimeFrame          = PERIOD_CURRENT; // Time frame to use
extern int                RsiPeriod          = 14;             // Rsi period
extern ENUM_APPLIED_PRICE RsiPrice           = PRICE_CLOSE;    // Rsi price
extern int                EmaSmoothing1      = 15;             // Ema smoothing period 1
extern int                EmaSmoothing2      =  3;             // Ema smoothing period 2
extern int                SmaSmoothing       =  9;             // Sma smoothing period
extern int                SignalPeriod       =  9;             // Signal period
extern bool               alertsOn           = false;          // Turn alerts on?
extern bool               alertsOnCurrent    = false;          // Alerts on current (still opened) bar?
extern bool               alertsMessage      = true;           // Alerts should show pop-up message?
extern bool               alertsNotification = false;          // Alerts should send notification?
extern bool               alertsSound        = false;          // Alerts should play alert sound?
extern bool               alertsEmail        = false;          // Alerts should send an email?
extern bool               Interpolate        = true;           // Interpolate in multi time frame mode?

//
//
//
//
//

double der[];
double derUu[];
double derUd[];
double derDd[];
double derDu[];
double trend[];
double signal[];
double slope[];

string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorDigits(5);
   IndicatorBuffers(8);
      SetIndexBuffer(0,derUu); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,derUd); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,derDu); SetIndexStyle(2,DRAW_HISTOGRAM);
      SetIndexBuffer(3,derDd); SetIndexStyle(3,DRAW_HISTOGRAM);
      SetIndexBuffer(4,der);
      SetIndexBuffer(5,signal);
      SetIndexBuffer(6,trend);
      SetIndexBuffer(7,slope);

      //
      //
      //
      //
      //
            
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame==-99);
      TimeFrame         = MathMax(TimeFrame,_Period);
   IndicatorShortName(timeFrameToString(TimeFrame)+" Derivative oscillator ("+(string)RsiPeriod+","+(string)EmaSmoothing1+","+(string)EmaSmoothing2+","+(string)SmaSmoothing+")");
   return(0);
}

//+------------------------------------------------------------------
//|
//+------------------------------------------------------------------
//
//
// 
//
// 
//
//

int start() 
{
   int counted_bars = IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
            int limit = MathMin(Bars-counted_bars,Bars-1);
            if (returnBars) { derUu[0] = limit; return(0); }
   	
   //
   //
   //
   //
   //
      	
   if (TimeFrame==Period())
   {      	
      for(int i=limit; i>=0; i--)
      {
         double rsi  = iRSI(NULL,0,RsiPeriod,RsiPrice,i);
         double ema1 = iEma(rsi ,EmaSmoothing1,i,0);
         double ema2 = iEma(ema1,EmaSmoothing2,i,1);
         double sma  = iSma(ema2,SmaSmoothing,i);

         //
         //
         //
         //
         //
      
         der[i]   = ema2-sma;
         derUu[i] = EMPTY_VALUE;
         derUd[i] = EMPTY_VALUE;
         derDu[i] = EMPTY_VALUE;
         derDd[i] = EMPTY_VALUE;
         signal[i] = iEma(der[i],SignalPeriod,i,2);
         if (i<Bars-1)
         {
            trend[i]  = trend[i+1];
            slope[i]  = slope[i+1];
               if (der[i]>0)        trend[i] =  1;
               if (der[i]<0)        trend[i] = -1;
               if (der[i]>der[i+1]) slope[i] =  1;
               if (der[i]<der[i+1]) slope[i] = -1;
               if (trend[i] == 1)
                  if (slope[i]==1)
                        derUu[i] = der[i];
                  else  derUd[i] = der[i];
               if (trend[i] == -1)
                  if (slope[i]==1)
                        derDu[i] = der[i];
                  else  derDd[i] = der[i];
         }                  
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   
   limit = (int)MathMin(Bars-1,MathMax(limit,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for(int i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         der[i]    = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,SignalPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,4,y);
         signal[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,SignalPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,5,y);
         trend[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,SignalPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,6,y);
         slope[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,RsiPeriod,RsiPrice,EmaSmoothing1,EmaSmoothing2,SmaSmoothing,SignalPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsNotification,alertsSound,alertsEmail,7,y);     
         derUu[i]  = EMPTY_VALUE;
         derUd[i]  = EMPTY_VALUE;
         derDu[i]  = EMPTY_VALUE;
         derDd[i]  = EMPTY_VALUE;
          
         //
         //
         //
         //
         //
      
         if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;

         //
         //
         //
         //
         //

         int n,k; datetime time = iTime(NULL,TimeFrame,y);
            for(n = 1; i+n<Bars && Time[i+n] >= time; n++) continue;	
            for(k = 1; i+n<Bars && i+k<Bars && k<n; k++)
            {
               der[i+k]    = der[i]    + (der[i+n]   -der[i]   )*k/n;
               signal[i+k] = signal[i] + (signal[i+n]-signal[i])*k/n;
            }               
   }              
   for(int i=limit; i>=0; i--)
   {
      if (trend[i] == 1)
               if (slope[i]==1)
                     derUu[i] = der[i];
               else  derUd[i] = der[i];
      if (trend[i] == -1)
               if (slope[i]==1)
                     derDu[i] = der[i];
               else  derDd[i] = der[i];
   }               
   return(0);
}



//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

double workSma[][2];
double iSma(double price, int period, int r, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workSma,0)!= Bars) ArrayResize(workSma,Bars); instanceNo *= 2; r = Bars-r-1; int k;

   //
   //
   //
   //
   //
      
   workSma[r][instanceNo+0] = price;
   workSma[r][instanceNo+1] = price; for(k=1; k<period && (r-k)>=0; k++) workSma[r][instanceNo+1] += workSma[r-k][instanceNo+0];  
   workSma[r][instanceNo+1] /= 1.0*k;
   return(workSma[r][instanceNo+1]);
}

//
//
//
//
//

double workEma[][3];
double iEma(double price, double period, int r, int instanceNo=0)
{
   if (period<=1) return(price);
   if (ArrayRange(workEma,0)!= Bars) ArrayResize(workEma,Bars); r = Bars-r-1;

   //
   //
   //
   //
   //
      
   workEma[r][instanceNo] = price;
   double alpha = 2.0 / (1.0+period);
   if (r>0)
          workEma[r][instanceNo] = workEma[r-1][instanceNo]+alpha*(price-workEma[r-1][instanceNo]);
   return(workEma[r][instanceNo]);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 1;
      
      //
      //
      //
      //
      //
      
      static datetime prevTime1  = 0;
      static string   prevAlert1 = "";
         if (trend[whichBar] != trend[whichBar+1])
         {
            if (trend[whichBar] ==  1) doAlert(prevTime1,prevAlert1," Derivative oscillator crossed zero line up");
            if (trend[whichBar] == -1) doAlert(prevTime1,prevAlert1," Derivative oscillator crossed zero line down");
         }
   }
}

//
//
//
//
//

void doAlert(datetime& previousTime, string& previousAlert, string doWhat)
{
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[0]) {
       previousAlert  = doWhat;
       previousTime   = Time[0];

       //
       //
       //
       //
       //

       message = timeFrameToString(Period())+" - "+Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+doWhat;
          if (alertsMessage)      Alert(message);
          if (alertsNotification) SendNotification(message);
          if (alertsEmail)        SendMail(StringConcatenate(Symbol(),"Derivative oscillator "),message);
          if (alertsSound)        PlaySound("alert2.wav");
   }
}